@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27019 $
+ * $Revision: 28153 $
  * $Author: rjongbloed $
- * $Date: 2012-02-20 20:44:54 -0600 (Mon, 20 Feb 2012) $
+ * $Date: 2012-08-06 22:45:21 -0500 (Mon, 06 Aug 2012) $
  */
 
 #ifdef __GNUC__
@@ -110,11 +110,6 @@ TRAVERSE_NODE(Choice);
 TRAVERSE_NODE(Property);
 TRAVERSE_NODE(Disconnect);
 TRAVERSE_NODE(Prompt);
-TRAVERSE_NODE(Filled);
-TRAVERSE_NODE(NoInput);
-TRAVERSE_NODE(NoMatch);
-TRAVERSE_NODE(Error);
-TRAVERSE_NODE(Catch);
 
 #define TRAVERSE_NODE2(name) \
   class PVXMLTraverse##name : public PVXMLNodeHandler { \
@@ -131,6 +126,25 @@ TRAVERSE_NODE2(Field);
 TRAVERSE_NODE2(Transfer);
 TRAVERSE_NODE2(Record);
 
+class PVXMLTraverseEvent : public PVXMLNodeHandler
+{
+  virtual bool Start(PVXMLSession &, PXMLElement & element) const
+  {
+    return element.GetAttribute("fired") == "true";
+  }
+
+  virtual bool Finish(PVXMLSession &, PXMLElement & element) const
+  {
+    element.SetAttribute("fired", "false");
+    return true;
+  }
+};
+static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> FilledNodeHandler("Filled", true);
+static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> NoInputNodeHandler("NoInput", true);
+static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> NoMatchNodeHandler("NoMatch", true);
+static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> ErrorNodeHandler("Error", true);
+static PVXMLNodeFactory::Worker<PVXMLTraverseEvent> CatchNodeHandler("Catch", true);
+
 #if PTRACING
 class PVXMLTraverseLog : public PVXMLNodeHandler {
   virtual bool Start(PVXMLSession & session, PXMLElement & node) const
@@ -827,23 +841,16 @@ PBoolean PVXMLSession::LoadFile(const PFilePath & filename, const PString & firs
 
 PBoolean PVXMLSession::LoadURL(const PURL & url)
 {
-  PTRACE(4, "VXML\tLoading URL: " << url);
+  PTRACE(4, "VXML\tLoading URL " << url);
 
   // retreive the document (may be a HTTP get)
-  PFilePath fn;
-  PString contentType;
-  if (!RetreiveResource(url, contentType, fn, false)) {
-    PTRACE(1, "VXML\tCannot load document " << url);
-    return false;
-  }
 
-  if (!LoadFile(fn, url.GetFragment())) {
-    PTRACE(1, "VXML\tCannot load VXML from " << url);
-    return false;
-  }
+  PString xmlStr;
+  if (url.LoadResource(xmlStr))
+    return LoadVXML(xmlStr, url.GetFragment());
 
-  m_rootURL = url;
-  return true;
+  PTRACE(1, "VXML\tCannot load document " << url);
+  return false;
 }
 
 
@@ -915,71 +922,50 @@ PBoolean PVXMLSession::RetreiveResource(const PURL & url,
                                      PFilePath & dataFn,
                                             PBoolean useCache)
 {
-  PBoolean stat = false;
-
   // files on the local file system get loaded locally
-  if (url.GetScheme() *= "file") {
+  if (url.GetScheme() == "file" && url.GetHostName().IsEmpty()) {
     dataFn = url.AsFilePath();
     if (contentType.IsEmpty())
       contentType = GetContentType(dataFn);
-    stat = true;
+    return true;
   }
 
-  // do a HTTP get when appropriate
-  else if ((url.GetScheme() *= "http") || (url.GetScheme() *= "https")) {
-
-    PFilePath fn;
-    PString fileType = url.AsFilePath().GetType();
-
-    PBoolean inCache = false;
-    if (useCache)
-      inCache = PVXMLCache::GetResourceCache().Get("url", url.AsString(), fileType, contentType, dataFn);
-
-    if (!inCache) {
-
-      // get a random filename
-      fn = PVXMLCache::GetResourceCache().GetRandomFilename("url", fileType);
-
-      // get the resource header information
-      PHTTPClient client;
-      PMIMEInfo outMIME, replyMIME;
-      if (!client.GetDocument(url, outMIME, replyMIME)) {
-        PTRACE(2, "VXML\tCannot load resource " << url);
-        stat =false;
-      }
-
-      else {
-
-        // Get the body of the response in a PBYTEArray (might be binary data)
-        PBYTEArray incomingData;
-        client.ReadContentBody(replyMIME, incomingData);
-        contentType = replyMIME(PHTTPClient::ContentTypeTag());
-
-        // write the data in the file
-        PFile cacheFile(fn, PFile::WriteOnly);
-        cacheFile.Write(incomingData.GetPointer(), incomingData.GetSize() );
-
-        // if we have a cache and we are using it, then save the data
-        if (useCache)
-          PVXMLCache::GetResourceCache().Put("url", url.AsString(), fileType, contentType, fn, dataFn);
-
-        // data is loaded
-        stat = true;
-      }
-    }
+  PString fileType;
+  {
+    const PStringArray & path = url.GetPath();
+    if (!path.IsEmpty())
+      fileType = PFilePath(path[path.GetSize()-1]).GetType();
   }
 
-  // files on the local file system get loaded locally
-  else if (url.GetScheme() *= "file") {
-    dataFn = url.AsFilePath();
-    stat = true;
+  if (useCache && PVXMLCache::GetResourceCache().Get("url", url.AsString(), fileType, contentType, dataFn))
+    return true;
+
+  // get a random filename
+  PFilePath newFn = PVXMLCache::GetResourceCache().GetRandomFilename("url", fileType);
+
+  // get the resource header information
+  PHTTPClient client;
+  PMIMEInfo outMIME, replyMIME;
+  if (!client.GetDocument(url, outMIME, replyMIME)) {
+    PTRACE(2, "VXML\tCannot load resource " << url);
+    return false;
   }
 
-  // unknown schemes give an error
-  else
-    stat = false;
+  // Get the body of the response in a PBYTEArray (might be binary data)
+  PBYTEArray incomingData;
+  client.ReadContentBody(replyMIME, incomingData);
+  contentType = replyMIME(PHTTPClient::ContentTypeTag());
 
-  return stat;
+  // write the data in the file
+  PFile cacheFile(newFn, PFile::WriteOnly);
+  cacheFile.Write(incomingData.GetPointer(), incomingData.GetSize());
+
+  // if we have a cache and we are using it, then save the data
+  if (useCache)
+    PVXMLCache::GetResourceCache().Put("url", url.AsString(), fileType, contentType, newFn, dataFn);
+
+  // data is loaded
+  return true;
 }
 
 
@@ -1014,6 +1000,20 @@ bool PVXMLSession::SetCurrentForm(const PString & searchId, bool fullURI)
               (id.IsEmpty() || (xmlElement->GetAttribute("id") *= id))
            ) {
           PTRACE(3, "VXML\tFound <" << xmlElement->GetName() << " id=\"" << xmlElement->GetAttribute("id") << "\">");
+
+          if (m_currentNode != NULL) {
+            PXMLElement * element = m_currentNode->GetParent();
+            while (element != NULL) {
+              PCaselessString nodeType = element->GetName();
+              PVXMLNodeHandler * handler = PVXMLNodeFactory::CreateInstance(nodeType);
+              if (handler != NULL) {
+                handler->Finish(*this, *element);
+                PTRACE(4, "VXML\tProcessed VoiceXML element: <" << nodeType << '>');
+              }
+              element = element->GetParent();
+            }
+          }
+
           m_currentNode = xmlObject;
           return true;
         }
@@ -1120,14 +1120,14 @@ void PVXMLSession::VXMLExecute(PThread &, INT)
       do {
         while (ProcessEvents())
           ;
-      } while (NextNode(false));
+      } while (NextNode(true));
     }
     else {
       // Wait till node finishes
       while (ProcessEvents())
         ;
 
-      NextNode(true);
+      NextNode(false);
     }
 
     // Determine if we should quit
@@ -1136,7 +1136,9 @@ void PVXMLSession::VXMLExecute(PThread &, INT)
 
     PTRACE(3, "VXML\tEnd of VoiceXML elements.");
 
+    m_sessionMutex.Signal();
     OnEndDialog();
+    m_sessionMutex.Wait();
 
     // Wait for anything OnEndDialog plays to complete.
     while (ProcessEvents())
@@ -1225,7 +1227,7 @@ bool PVXMLSession::ProcessEvents()
 }
 
 
-bool PVXMLSession::NextNode(bool skipChildren)
+bool PVXMLSession::NextNode(bool processChildren)
 {
   // m_sessionMutex already locked
 
@@ -1239,16 +1241,13 @@ bool PVXMLSession::NextNode(bool skipChildren)
   if (m_xmlChanged)
     return false;
 
-  // Skip all children
-  if (skipChildren)
-    m_currentNode = m_currentNode->GetNextObject();
-
   PXMLElement * element;
 
   if (m_currentNode->IsElement()) {
     element = (PXMLElement*)m_currentNode;
+
     // if the current node has children, then process the first child
-    if ((m_currentNode = element->GetElement(0)) != NULL)
+    if (processChildren && (m_currentNode = element->GetElement(0)) != NULL)
       return false;
   }
   else {
@@ -1269,8 +1268,10 @@ bool PVXMLSession::NextNode(bool skipChildren)
     PCaselessString nodeType = element->GetName();
     PVXMLNodeHandler * handler = PVXMLNodeFactory::CreateInstance(nodeType);
     if (handler != NULL) {
-      if (!handler->Finish(*this, *element))
+      if (!handler->Finish(*this, *element)) {
+        PTRACE(4, "VXML\tContinue processing VoiceXML element: <" << nodeType << '>');
         return true;
+      }
       PTRACE(4, "VXML\tProcessed VoiceXML element: <" << nodeType << '>');
     }
 
@@ -1286,7 +1287,7 @@ bool PVXMLSession::NextNode(bool skipChildren)
 bool PVXMLSession::ProcessGrammar()
 {
   if (m_grammar == NULL) {
-    PTRACE(1, "VXML\tNo grammar was created!");
+    PTRACE(4, "VXML\tNo grammar was created!");
     return true;
   }
 
@@ -2246,44 +2247,6 @@ PBoolean PVXMLSession::TraversedField(PXMLElement &)
 }
 
 
-static bool HasFired(PXMLElement & element)
-{
-  bool fired = element.GetAttribute("fired") == "true";
-  element.SetAttribute("fired", "false");
-  return fired;
-}
-
-
-PBoolean PVXMLSession::TraverseFilled(PXMLElement & element)
-{
-  return HasFired(element);
-}
-
-
-PBoolean PVXMLSession::TraverseNoInput(PXMLElement & element)
-{
-  return HasFired(element);
-}
-
-
-PBoolean PVXMLSession::TraverseNoMatch(PXMLElement & element)
-{
-  return HasFired(element);
-}
-
-
-PBoolean PVXMLSession::TraverseError(PXMLElement & element)
-{
-  return HasFired(element);
-}
-
-
-PBoolean PVXMLSession::TraverseCatch(PXMLElement & element)
-{
-  return HasFired(element);
-}
-
-
 void PVXMLSession::OnEndRecording()
 {
   //SetVar(channelName + ".size", PString(incomingChannel->GetWAVFile()->GetDataLength() ) );
