@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26573 $
+ * $Revision: 28049 $
  * $Author: rjongbloed $
- * $Date: 2011-10-10 05:54:20 -0500 (Mon, 10 Oct 2011) $
+ * $Date: 2012-07-18 02:15:00 -0500 (Wed, 18 Jul 2012) $
  */
 
 #include <ptlib/socket.h>
@@ -77,7 +77,7 @@ static PBoolean PAssertThreadOp(int retval,
 {
   if (retval == 0) {
     PTRACE_IF(2, retry > 0, "PTLib\t" << funcname << " required " << retry << " retries!");
-    return PFalse;
+    return false;
   }
 
   if (errno == EINTR || errno == EAGAIN) {
@@ -87,7 +87,7 @@ static PBoolean PAssertThreadOp(int retval,
 #else
       usleep(10000); // Basically just swap out thread to try and clear blockage
 #endif
-      return PTrue;   // Return value to try again
+      return true;   // Return value to try again
     }
     // Give up and assert
   }
@@ -95,7 +95,7 @@ static PBoolean PAssertThreadOp(int retval,
 #if P_USE_ASSERTS
   PAssertFunc(file, line, NULL, psprintf("Function %s failed", funcname));
 #endif
-  return PFalse;
+  return false;
 }
 
 
@@ -147,10 +147,10 @@ PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
       : PThread(1000, NoAutoDeleteThread, HighestPriority, "Housekeeper")
-      { closing = PFalse; Resume(); }
+      { closing = false; Resume(); }
 
     void Main();
-    void SetClosing() { closing = PTrue; }
+    void SetClosing() { closing = true; }
 
   protected:
     PBoolean closing;
@@ -180,13 +180,13 @@ void PHouseKeepingThread::Main()
       found = false;
       for (PProcess::ThreadMap::iterator it = process.m_activeThreads.begin(); it != process.m_activeThreads.end(); ++it) {
         PThread * thread = it->second;
-        if (thread->autoDelete && thread->IsTerminated()) {
+        if (thread->IsAutoDelete() && thread->IsTerminated()) {
           process.m_activeThreads.erase(it);
 
           // unlock the m_activeThreadMutex to avoid deadlocks:
           // if somewhere in the destructor a call to PTRACE() is made,
           // which itself calls PThread::Current(), deadlocks are possible
-          thread->PX_threadId = 0;
+          thread->m_threadId = 0;
           process.m_activeThreadMutex.Signal();
           delete thread;
           process.m_activeThreadMutex.Wait();
@@ -207,13 +207,13 @@ void PHouseKeepingThread::Main()
 
 bool PProcess::SignalTimerChange()
 {
-  if (m_shuttingDown)
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
     return false;
 
   PWaitAndSignal m(housekeepingMutex);
   if (housekeepingThread == NULL) {
 #if PMEMORY_CHECK
-    PBoolean oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(PTrue);
+    PBoolean oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(true);
 #endif
     housekeepingThread = new PHouseKeepingThread;
 #if PMEMORY_CHECK
@@ -264,14 +264,14 @@ PBoolean PProcess::SetMaxHandles(int newMax)
     maxHandles = rl.rlim_cur;
     if (maxHandles == newMax) {
       PTRACE(2, "PTLib\tNew maximum per-process file handles set to " << maxHandles);
-      return PTrue;
+      return true;
     }
   }
 #endif // !P_RTEMS
 
   PTRACE(1, "PTLib\tCannot set per-process file handle limit to "
          << newMax << " (is " << maxHandles << ") - check permissions");
-  return PFalse;
+  return false;
 }
 
 
@@ -312,7 +312,7 @@ void PProcess::PXSetThread(pthread_t id, PThread * thread)
   m_activeThreadMutex.Wait();
 
   ThreadMap::iterator it = m_activeThreads.find(id);
-  if (it != m_activeThreads.end() && it->second->autoDelete)
+  if (it != m_activeThreads.end() && it->second->IsAutoDelete())
     currentThread = it->second;
 
   m_activeThreads[id] = thread;
@@ -335,43 +335,34 @@ void PProcess::PXSetThread(pthread_t id, PThread * thread)
 //  is not paused
 //
 
-PThread::PThread()
-{
-  autoDelete          = PFalse;
-
-  // PX_origStackSize = 0 indicates external thread
-  PX_origStackSize    = 0;
-  PX_threadId         = pthread_self();
+PThread::PThread(bool isProcess)
+  : m_isProcess(isProcess)
+  , m_autoDelete(!isProcess)
+  , m_originalStackSize(0) // 0 indicates external thread
+  , m_threadId(pthread_self())
+  , PX_priority(NormalPriority)
 #if defined(P_LINUX)
-  PX_linuxId          = syscall(SYS_gettid);
+  , PX_linuxId(syscall(SYS_gettid))
 #endif
-  PX_priority         = NormalPriority;
-  PX_suspendCount     = 0;
-
+  , PX_suspendMutex(MutexInitialiser)
+  , PX_suspendCount(0)
+  , PX_firstTimeStart(false)
 #ifndef P_HAS_SEMAPHORES
-  PX_waitingSemaphore = NULL;
-  PX_WaitSemMutex = MutexInitialiser;
+  , PX_waitingSemaphore(NULL)
+  , PX_WaitSemMutex(MutexInitialiser)
 #endif
-
-  PX_suspendMutex = MutexInitialiser;
-
+{
 #ifdef P_RTEMS
   PAssertOS(socketpair(AF_INET,SOCK_STREAM,0,unblockPipe) == 0);
 #else
   PAssertOS(::pipe(unblockPipe) == 0);
 #endif
 
-  PX_firstTimeStart = PFalse;
-
-  if (!PProcess::IsInitialised())
+  if (isProcess)
     return;
 
-  autoDelete = true;
-
   PProcess & process = PProcess::Current();
-
-  process.PXSetThread(PX_threadId, this);
-
+  process.PXSetThread(m_threadId, this);
   process.SignalTimerChange();
 }
 
@@ -386,29 +377,24 @@ PThread::PThread(PINDEX stackSize,
                  AutoDeleteFlag deletion,
                  Priority priorityLevel,
                  const PString & name)
-  : threadName(name)
-{
-  autoDelete = (deletion == AutoDeleteThread);
-
-  PAssert(stackSize > 0, PInvalidParameter);
-  PX_priority = priorityLevel;
-  PX_suspendCount = 1;
-
-  // PX_origStackSize != 0 indicates PTLib created thread
-  PX_origStackSize = stackSize;
-
-  // PX_threadId = 0 indicates thread has not started
-  PX_threadId = 0;                          
+  : m_isProcess(false)
+  , m_autoDelete(deletion == AutoDeleteThread)
+  , m_originalStackSize(stackSize) // 0 indicates PTLib created thread
+  , m_threadName(name)
+  , m_threadId(0)  // 0 indicates thread has not started
+  , PX_priority(priorityLevel)
 #if defined(P_LINUX)
-  PX_linuxId = 0;
+  , PX_linuxId(0)
 #endif
-
+  , PX_suspendMutex(MutexInitialiser)
+  , PX_suspendCount(1)
+  , PX_firstTimeStart(true) // new thread is actually started the first time Resume() is called.
 #ifndef P_HAS_SEMAPHORES
-  PX_waitingSemaphore = NULL;
-  PX_WaitSemMutex = MutexInitialiser;
+  , PX_waitingSemaphore(NULL)
+  , PX_WaitSemMutex(MutexInitialiser)
 #endif
-
-  PX_suspendMutex = MutexInitialiser;
+{
+  PAssert(stackSize > 0, PInvalidParameter);
 
 #ifdef P_RTEMS
   PAssertOS(socketpair(AF_INET,SOCK_STREAM,0,unblockPipe) == 0);
@@ -417,10 +403,7 @@ PThread::PThread(PINDEX stackSize,
 #endif
   PX_NewHandle("Thread unblock pipe", PMAX(unblockPipe[0], unblockPipe[1]));
 
-  // new thread is actually started the first time Resume() is called.
-  PX_firstTimeStart = PTrue;
-
-  PTRACE(5, "PTLib\tCreated thread " << this << ' ' << threadName);
+  PTRACE(5, "PTLib\tCreated thread " << this << ' ' << m_threadName);
 }
 
 //
@@ -437,7 +420,7 @@ PThread::~PThread()
     PTrace::Cleanup();
 #endif
   } else {
-    pthread_t id = PX_threadId;
+    pthread_t id = m_threadId;
     PProcess & process = PProcess::Current();
 
     // need to terminate the thread if it was ever started and it is not us
@@ -448,13 +431,13 @@ PThread::~PThread()
     process.SignalTimerChange();
 
     // last gasp tracing
-    PTRACE(5, "PTLib\tDestroyed thread " << this << ' ' << threadName << "(id = " << ::hex << id << ::dec << ")");
+    PTRACE(5, "PTLib\tDestroyed thread " << this << ' ' << m_threadName << "(id = " << ::hex << id << ::dec << ")");
 
 
     // if thread was started, remove it from the active thread list and detach it to release thread resources
     if (id != 0) {
       process.m_activeThreadMutex.Wait();
-      if (PX_origStackSize != 0)
+      if (m_originalStackSize != 0)
         pthread_detach(id);
       process.m_activeThreads.erase(id);
       process.m_activeThreadMutex.Signal();
@@ -483,7 +466,7 @@ void * PThread::PX_ThreadStart(void * arg)
 { 
   PThread * thread = (PThread *)arg;
   // Added this to guarantee that the thread creation (PThread::Restart)
-  // has completed before we start the thread. Then the PX_threadId has
+  // has completed before we start the thread. Then the m_threadId has
   // been set.
   pthread_mutex_lock(&thread->PX_suspendMutex);
   thread->SetThreadName(thread->GetThreadName());
@@ -517,7 +500,8 @@ void * PThread::PX_ThreadStart(void * arg)
 #endif
 
   // Inform the helgrind finite state machine that this thread has finished
-  pthread_exit(0);
+  // Commented out as on some platforms it causes a crash, no idea why!
+  // pthread_exit(0);
 
   return NULL;
 }
@@ -533,10 +517,6 @@ void PThread::PX_ThreadEnd(void * arg)
 #endif
 
   process.OnThreadEnded(*thread);
-  
-  bool deleteThread = thread->autoDelete; // make copy of the flag before perhaps deleting the thread
-  if (deleteThread)
-    delete thread;
 }
 
 
@@ -567,23 +547,27 @@ void PThread::Restart()
 #endif
 
   PProcess & process = PProcess::Current();
-  size_t newHighWaterMark = 0;
-  static size_t highWaterMark = 0;
 
   // lock the thread list
   process.m_activeThreadMutex.Wait();
 
   // create the thread
-  PAssertPTHREAD(pthread_create, (&PX_threadId, &threadAttr, PX_ThreadStart, this));
+  PAssertPTHREAD(pthread_create, (&m_threadId, &threadAttr, PX_ThreadStart, this));
 
   // put the thread into the thread list
-  process.PXSetThread(PX_threadId, this);
+  process.PXSetThread(m_threadId, this);
+
+  // Inside process.m_activeThreadMutex so simple static is OK
+  size_t newHighWaterMark = 0;
+  static size_t highWaterMark = 0;
   if (process.m_activeThreads.size() > highWaterMark)
     newHighWaterMark = highWaterMark = process.m_activeThreads.size();
 
   // unlock the thread list
   process.m_activeThreadMutex.Signal();
 
+  pthread_attr_destroy(&threadAttr);
+
   PTRACE_IF(newHighWaterMark%100 == 0 ? 2 : 4, newHighWaterMark > 0,
             "PTLib\tThread high water mark set: " << newHighWaterMark);
 
@@ -602,7 +586,7 @@ void PX_SuspendSignalHandler(int)
   if (thread == NULL)
     return;
 
-  PBoolean notResumed = PTrue;
+  PBoolean notResumed = true;
   while (notResumed) {
     BYTE ch;
     notResumed = ::read(thread->unblockPipe[0], &ch, 1) < 0 && errno == EINTR;
@@ -625,7 +609,7 @@ void PThread::Suspend(PBoolean susp)
       if (PX_suspendCount > 0)
         PX_suspendCount--;
       if (PX_suspendCount == 0) {
-        PX_firstTimeStart = PFalse;
+        PX_firstTimeStart = false;
         Restart();
       }
     }
@@ -638,15 +622,15 @@ void PThread::Suspend(PBoolean susp)
   // Suspend - warn the user with an Assertion
   PAssertAlways("Cannot suspend threads on Mac OS X due to lack of pthread_kill()");
 #else
-  if (PPThreadKill(PX_threadId, 0)) {
+  if (PPThreadKill(m_threadId, 0)) {
 
     // if suspending, then see if already suspended
     if (susp) {
       PX_suspendCount++;
       if (PX_suspendCount == 1) {
-        if (PX_threadId != pthread_self()) {
+        if (m_threadId != pthread_self()) {
           signal(SUSPEND_SIG, PX_SuspendSignalHandler);
-          PPThreadKill(PX_threadId, SUSPEND_SIG);
+          PPThreadKill(m_threadId, SUSPEND_SIG);
         }
         else {
           PAssertPTHREAD(pthread_mutex_unlock, (&PX_suspendMutex));
@@ -671,17 +655,17 @@ void PThread::Suspend(PBoolean susp)
 
 void PThread::Resume()
 {
-  Suspend(PFalse);
+  Suspend(false);
 }
 
 
 PBoolean PThread::IsSuspended() const
 {
   if (PX_firstTimeStart)
-    return PTrue;
+    return true;
 
   if (IsTerminated())
-    return PFalse;
+    return false;
 
   PAssertPTHREAD(pthread_mutex_lock, ((pthread_mutex_t *)&PX_suspendMutex));
   PBoolean suspended = PX_suspendCount != 0;
@@ -692,8 +676,8 @@ PBoolean PThread::IsSuspended() const
 
 void PThread::SetAutoDelete(AutoDeleteFlag deletion)
 {
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
-  autoDelete = deletion == AutoDeleteThread;
+  PAssert(deletion != AutoDeleteThread || (!m_isProcess && this != &PProcess::Current()), PLogicError);
+  m_autoDelete = deletion == AutoDeleteThread;
 }
 
 #ifdef P_MACOSX
@@ -754,7 +738,7 @@ void PThread::SetPriority(Priority priorityLevel)
 
 #if defined(P_LINUX)
   struct sched_param params;
-  PAssertPTHREAD(pthread_setschedparam, (PX_threadId, GetSchedParam(priorityLevel, params), &params));
+  PAssertPTHREAD(pthread_setschedparam, (m_threadId, GetSchedParam(priorityLevel, params), &params));
 
 #elif defined(P_MACOSX)
   if (priorityLevel == HighestPriority) {
@@ -767,7 +751,7 @@ void PThread::SetPriority(Priority priorityLevel)
       long                            relativePriority;
 
       theFixedPolicy.timeshare = false; // set to true for a non-fixed thread
-      result = thread_policy_set (pthread_mach_thread_np(PX_threadId),
+      result = thread_policy_set (pthread_mach_thread_np(m_threadId),
                                   THREAD_EXTENDED_POLICY,
                                   (thread_policy_t)&theFixedPolicy,
                                   THREAD_EXTENDED_POLICY_COUNT);
@@ -784,7 +768,7 @@ void PThread::SetPriority(Priority priorityLevel)
       PTRACE(3,  "relativePriority is " << relativePriority << " base priority is " << GetThreadBasePriority());
       
       thePrecedencePolicy.importance = relativePriority;
-      result = thread_policy_set (pthread_mach_thread_np(PX_threadId),
+      result = thread_policy_set (pthread_mach_thread_np(m_threadId),
                                   THREAD_PRECEDENCE_POLICY,
                                   (thread_policy_t)&thePrecedencePolicy, 
                                   THREAD_PRECEDENCE_POLICY_COUNT);
@@ -803,7 +787,7 @@ PThread::Priority PThread::GetPriority() const
   int policy;
   struct sched_param params;
   
-  PAssertPTHREAD(pthread_getschedparam, (PX_threadId, &policy, &params));
+  PAssertPTHREAD(pthread_getschedparam, (m_threadId, &policy, &params));
   
   switch (policy)
   {
@@ -893,13 +877,13 @@ void PThread::Yield()
 void PThread::Terminate()
 {
   // if thread was not created by PTLib, then don't terminate it
-  if (PX_origStackSize <= 0)
+  if (m_originalStackSize <= 0)
     return;
 
   // if thread calls Terminate on itself, then do it
   // don't use PThread::Current, as the thread may already not be in the
   // active threads list
-  if (PX_threadId == pthread_self()) {
+  if (m_threadId == pthread_self()) {
     pthread_exit(0);
     return;   // keeps compiler happy
   }
@@ -926,30 +910,26 @@ void PThread::Terminate()
 #endif
 
 #if ( defined(P_NETBSD) && defined(P_NO_CANCEL) )
-  PPThreadKill(PX_threadId,SIGKILL);
+  PPThreadKill(m_threadId, SIGKILL);
 #else
-  if (PX_threadId) {
-    pthread_cancel(PX_threadId);
+  if (m_threadId) {
+    pthread_cancel(m_threadId);
   }
 #endif
 }
 
 
-//  
-//  See if thread is still running
-//  Note PX_threadId = 0 means thread has been created but not yet started
-//
 PBoolean PThread::IsTerminated() const
 {
-  pthread_t id = PX_threadId;
-  return (id == 0) || (PX_origStackSize <= 0) || (pthread_kill(id, 0) != 0);
+  if (m_isProcess)
+    return false; // Process is always still running
+
+  // See if thread is still running
+  pthread_t id = m_threadId;
+  return id == 0 || pthread_kill(id, 0) != 0;
 }
 
 
-//  
-//  Wait for thread to terminate
-//  
-//
 void PThread::WaitForTermination() const
 {
   WaitForTermination(PMaxTimeInterval);
@@ -958,8 +938,8 @@ void PThread::WaitForTermination() const
 
 PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 {
-  pthread_t id = PX_threadId;
-  if ((id == 0) || (id == Current()->GetThreadId())) {
+  pthread_t id = m_threadId;
+  if (id == 0 || this == Current()) {
     PTRACE(2, "WaitForTermination on 0x" << hex << id << dec << " short circuited");
     return true;
   }
@@ -1371,7 +1351,7 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
 {
   if (waitTime == PMaxTimeInterval) {
     Wait();
-    return PTrue;
+    return true;
   }
 
   // create absolute finish time 
@@ -1401,7 +1381,7 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
   // thread to get very busy
   do {
     if (sem_trywait(&semId) == 0)
-      return PTrue;
+      return true;
 
 #if defined(P_LINUX)
   // sched_yield in a tight loop is bad karma
@@ -1412,17 +1392,17 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
 #endif
   } while (PTime() < finishTime);
 
-  return PFalse;
+  return false;
 
 #endif
 #elif defined(P_HAS_NAMED_SEMAPHORES)
   do {
     if(sem_trywait(semId) == 0)
-      return PTrue;
+      return true;
     PThread::Current()->Sleep(10);
   } while (PTime() < finishTime);
   
-  return PFalse;
+  return false;
 #else
 
   struct timespec absTime;
@@ -1435,11 +1415,11 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
   thread->PXSetWaitingSemaphore(this);
   queuedLocks++;
 
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
   while (currentCount == 0) {
     int err = pthread_cond_timedwait(&condVar, &mutex, &absTime);
     if (err == ETIMEDOUT) {
-      ok = PFalse;
+      ok = false;
       break;
     }
     else
@@ -1484,26 +1464,40 @@ PBoolean PSemaphore::WillBlock() const
 #if defined(P_HAS_SEMAPHORES)
   if (sem_trywait((sem_t *)&semId) != 0) {
     PAssertOS(errno == EAGAIN || errno == EINTR);
-    return PTrue;
+    return true;
   }
   PAssertPTHREAD(sem_post, ((sem_t *)&semId));
-  return PFalse;
+  return false;
 #elif defined(P_HAS_NAMED_SEMAPHORES)
   if (sem_trywait(semId) != 0) {
     PAssertOS(errno == EAGAIN || errno == EINTR);
-    return PTrue;
+    return true;
   }
   PAssertPTHREAD(sem_post, (semId));
-  return PFalse;
+  return false;
 #else
   return currentCount == 0;
 #endif
 }
 
+
+///////////////////////////////////////////////////////////////////////////////
+
 PTimedMutex::PTimedMutex()
-//  : PSemaphore(PXMutex)
+{
+  Construct();
+}
+
+PTimedMutex::PTimedMutex(const PTimedMutex &)
+{
+  Construct();
+}
+
+
+void PTimedMutex::Construct()
 {
 #if P_HAS_RECURSIVE_MUTEX
+
   pthread_mutexattr_t attr;
   PAssertPTHREAD(pthread_mutexattr_init, (&attr));
 
@@ -1513,94 +1507,90 @@ PTimedMutex::PTimedMutex()
   PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
 #endif
 
-  PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
+  PAssertPTHREAD(pthread_mutex_init, (&m_mutex, &attr));
   PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
-#else
-  PAssertPTHREAD(pthread_mutex_init, (&mutex, NULL));
-#endif
+
+#else // P_HAS_RECURSIVE_MUTEX
+
+  m_lockerId = (pthread_id)-1;
+  PAssertPTHREAD(pthread_mutex_init, (&m_mutex, NULL));
+
+#endif // P_HAS_RECURSIVE_MUTEX
 }
 
-PTimedMutex::PTimedMutex(const PTimedMutex & /*mut*/)
-//  : PSemaphore(PXMutex)
-{
-#if P_HAS_RECURSIVE_MUTEX
-  pthread_mutexattr_t attr;
-  PAssertPTHREAD(pthread_mutexattr_init, (&attr));
-#if P_HAS_RECURSIVE_MUTEX == 2
-  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE));
-#else
-  PAssertPTHREAD(pthread_mutexattr_settype, (&attr, PTHREAD_MUTEX_RECURSIVE_NP));
-#endif
-
-  PAssertPTHREAD(pthread_mutex_init, (&mutex, &attr));
-  PAssertPTHREAD(pthread_mutexattr_destroy, (&attr));
-#else
-  pthread_mutex_init(&mutex, NULL);
-#endif
-}
 
 PTimedMutex::~PTimedMutex()
 {
-  int result = pthread_mutex_destroy(&mutex);
-  PINDEX i = 0;
-  while ((result == EBUSY) && (i++ < 20)) {
-    pthread_mutex_unlock(&mutex);
-    result = pthread_mutex_destroy(&mutex);
+  int result = pthread_mutex_destroy(&m_mutex);
+  if (result == EBUSY) {
+    // In case it is us
+    while (pthread_mutex_unlock(&m_mutex) == 0)
+      ;
+
+    // Wait a bit for someone else to unlock it
+    for (PINDEX i = 0; i < 100; ++i) {
+      if ((result = pthread_mutex_destroy(&m_mutex)) != EBUSY)
+        break;
+      pthread_yield();
+    }
   }
+
 #ifdef _DEBUG
-  PAssert((result == 0), "Error destroying mutex");
+  PAssert(result == 0, "Error destroying mutex");
 #endif
 }
 
+
 void PTimedMutex::Wait() 
 {
-  pthread_t currentThreadId = pthread_self();
+#if P_HAS_RECURSIVE_MUTEX
 
-#if P_HAS_RECURSIVE_MUTEX == 0
+  PAssertPTHREAD(pthread_mutex_lock, (&m_mutex));
+
+#else //P_HAS_RECURSIVE_MUTEX
+
+  pthread_t currentThreadId = pthread_self();
 
   // if the mutex is already acquired by this thread,
   // then just increment the lock count
-  if (pthread_equal(lockerId, currentThreadId)) {
+  if (pthread_equal(m_lockerId, currentThreadId)) {
     // Note this does not need a lock as it can only be touched by the thread
     // which already has the mutex locked.
-    ++lockCount;
+    ++m_lockCount;
     return;
   }
-#endif
 
   // acquire the lock for real
-  PAssertPTHREAD(pthread_mutex_lock, (&mutex));
+  PAssertPTHREAD(pthread_mutex_lock, (&m_mutex));
 
-#if P_HAS_RECURSIVE_MUTEX == 0
-  PAssert((lockerId == (pthread_t)-1) && (lockCount.IsZero()),
+  PAssert(m_lockerId == (pthread_t)-1 && m_lockCount.IsZero(),
           "PMutex acquired whilst locked by another thread");
+
   // Note this is protected by the mutex itself only the thread with
   // the lock can alter it.
-#endif
+  m_lockerId = currentThreadId;
 
-  lockerId = currentThreadId;
+#endif // P_HAS_RECURSIVE_MUTEX
 }
 
 
 PBoolean PTimedMutex::Wait(const PTimeInterval & waitTime) 
 {
-  // get the current thread ID
-  pthread_t currentThreadId = pthread_self();
-
   // if waiting indefinitely, then do so
   if (waitTime == PMaxTimeInterval) {
     Wait();
-    lockerId = currentThreadId;
-    return PTrue;
+    return true;
   }
 
-#if P_HAS_RECURSIVE_MUTEX == 0
+#if !P_HAS_RECURSIVE_MUTEX
+  pthread_t currentThreadId = pthread_self();
+
   // if we already have the mutex, return immediately
-  if (pthread_equal(lockerId, currentThreadId)) {
+  if (pthread_equal(m_lockerId, currentThreadId)) {
     // Note this does not need a lock as it can only be touched by the thread
     // which already has the mutex locked.
-    ++lockCount;
-    return PTrue;
+    ++m_lockCount;
+    return true;
   }
 #endif
 
@@ -1614,45 +1604,36 @@ PBoolean PTimedMutex::Wait(const PTimeInterval & waitTime)
   absTime.tv_sec  = finishTime.GetTimeInSeconds();
   absTime.tv_nsec = finishTime.GetMicrosecond() * 1000;
 
-  if (pthread_mutex_timedlock(&mutex, &absTime) != 0)
-    return PFalse;
-
-#if P_HAS_RECURSIVE_MUTEX == 0
-  PAssert((lockerId == (pthread_t)-1) && (lockCount.IsZero()),
-          "PMutex acquired whilst locked by another thread");
-#endif
-
-  // Note this is protected by the mutex itself only the thread with
-  // the lock can alter it.
-  lockerId = currentThreadId;
-  return PTrue;
+  if (pthread_mutex_timedlock(&m_mutex, &absTime) != 0)
+    return false;
 
 #else // P_PTHREADS_XPG6
 
-  for (;;) {
-    if (pthread_mutex_trylock(&mutex) == 0) {
-#if P_HAS_RECURSIVE_MUTEX == 0
-      PAssert((lockerId == (pthread_t)-1) && (lockCount.IsZero()),
-              "PMutex acquired whilst locked by another thread");
-#endif
-      lockerId = currentThreadId;
-
-      return PTrue;
-    }
-
+  while (pthread_mutex_trylock(&m_mutex) != 0) {
     if (PTime() >= finishTime)
-      return PFalse;
-
-    PThread::Current()->Sleep(10); // sleep for 10ms
+      return false;
+    usleep(10000);
   }
+
 #endif // P_PTHREADS_XPG6
+
+#if !P_HAS_RECURSIVE_MUTEX
+  PAssert((lockerId == (pthread_t)-1) && m_lockCount.IsZero(),
+          "PMutex acquired whilst locked by another thread");
+
+  // Note this is protected by the mutex itself only the thread with
+  // the lock can alter it.
+  m_lockerId = currentThreadId;
+#endif
+
+  return true;
 }
 
 
 void PTimedMutex::Signal()
 {
-#if P_HAS_RECURSIVE_MUTEX == 0
-  if (!pthread_equal(lockerId, pthread_self())) {
+#if !P_HAS_RECURSIVE_MUTEX
+  if (!pthread_equal(m_lockerId, pthread_self())) {
     PAssertAlways("PMutex signal failed - no matching wait or signal by wrong thread");
     return;
   }
@@ -1660,37 +1641,37 @@ void PTimedMutex::Signal()
   // if lock was recursively acquired, then decrement the counter
   // Note this does not need a separate lock as it can only be touched by the thread
   // which already has the mutex locked.
-  if (!lockCount.IsZero()) {
-    --lockCount;
+  if (!m_lockCount.IsZero()) {
+    --m_lockCount;
     return;
   }
 
   // otherwise mark mutex as available
-  lockerId = (pthread_t)-1;
+  m_lockerId = (pthread_t)-1;
 
 #endif
 
-  PAssertPTHREAD(pthread_mutex_unlock, (&mutex));
+  PAssertPTHREAD(pthread_mutex_unlock, (&m_mutex));
 }
 
 
 PBoolean PTimedMutex::WillBlock() const
 {
-#if P_HAS_RECURSIVE_MUTEX == 0
-  pthread_t currentThreadId = pthread_self();
-  if (currentThreadId == lockerId)
-    return PFalse;
+#if !P_HAS_RECURSIVE_MUTEX
+  if (pthread_equal(m_lockerId, pthread_self()))
+    return false;
 #endif
 
-  pthread_mutex_t * mp = (pthread_mutex_t*)&mutex;
-  if (pthread_mutex_trylock(mp) != 0)
-    return PTrue;
+  if (pthread_mutex_trylock(&m_mutex) != 0)
+    return true;
 
-  PAssertPTHREAD(pthread_mutex_unlock, (mp));
-  return PFalse;
+  PAssertPTHREAD(pthread_mutex_unlock, (&m_mutex));
+  return false;
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
 PSyncPoint::PSyncPoint()
   : PSemaphore(PXSyncPoint)
 {
